"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDebugAsyncBrowser = exports.debugAsync = void 0;
const prepare_debug_async_1 = require("./prepare-debug-async");
const core_1 = require("./core");
const extend_context_1 = require("./extend-context");
exports.debugAsync = createDebugAsyncBrowser();
function createDebugAsyncBrowser({ defaults: { contexts: contextsDefault = [globalThis], overrideProperties: overridePropertiesDefault = true, apiNamespace: apiNamespaceDefault = core_1.API_NAMESPACE, logger: loggerDefault, } = {}, } = {}) {
    let isBeingDebugged = false;
    return async (bindings, options = {}) => {
        if (isBeingDebugged) {
            // Better to just ignore.
            return;
        }
        const { contexts = contextsDefault, overrideProperties = overridePropertiesDefault, apiNamespace = apiNamespaceDefault, logger: loggerOptions = loggerDefault, } = options;
        const logger = (0, core_1.getLogger)(loggerOptions);
        const { resultPromise, extension, startMessage, stopMessage } = (0, prepare_debug_async_1.prepareDebugAsync)(bindings, apiNamespace);
        logger.info?.(startMessage);
        const cleanups = contexts.map((context) => (0, extend_context_1.extendContext)(context, extension, overrideProperties, logger));
        try {
            isBeingDebugged = true;
            return await resultPromise;
        }
        finally {
            isBeingDebugged = false;
            for (const cleanup of cleanups) {
                cleanup();
            }
            logger.info?.(stopMessage);
        }
    };
}
exports.createDebugAsyncBrowser = createDebugAsyncBrowser;
