"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDebugAsyncNode = exports.debugAsync = void 0;
const repl_1 = require("repl");
const prepare_debug_async_1 = require("./prepare-debug-async");
const core_1 = require("./core");
const extend_context_1 = require("./extend-context");
exports.debugAsync = createDebugAsyncNode();
function createDebugAsyncNode({ defaults: { contexts: contextsDefault = [globalThis], overrideProperties: overridePropertiesDefault = true, apiNamespace: apiNamespaceDefault = core_1.API_NAMESPACE, logger: loggerDefault, replOptions: replOptionsDefault, } = {}, } = {}) {
    let isBeingDebugged = false;
    return async (bindings, options = {}) => {
        if (isBeingDebugged) {
            // Better to just ignore.
            return;
        }
        const { contexts = contextsDefault, overrideProperties = overridePropertiesDefault, apiNamespace = apiNamespaceDefault, logger: loggerOptions = loggerDefault, replOptions = replOptionsDefault, } = options;
        const logger = (0, core_1.getLogger)(loggerOptions);
        const { resultPromise, extension, api, startMessage, stopMessage } = (0, prepare_debug_async_1.prepareDebugAsync)(bindings, apiNamespace);
        logger.info?.(startMessage);
        const cleanups = contexts.map((context) => (0, extend_context_1.extendContext)(context, extension, overrideProperties, logger));
        const repl = (0, repl_1.start)(replOptions);
        const exitPromise = new Promise((resolve) => {
            repl.once('exit', () => {
                api.resumeExecution();
                resolve();
            });
        });
        try {
            (0, extend_context_1.extendContext)(repl.context, extension, true, { ...logger, warn: null });
            isBeingDebugged = true;
            return await resultPromise;
        }
        finally {
            repl.close();
            await exitPromise;
            isBeingDebugged = false;
            for (const cleanup of cleanups) {
                cleanup();
            }
            logger.info?.(stopMessage);
        }
    };
}
exports.createDebugAsyncNode = createDebugAsyncNode;
