"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareDebugAsync = void 0;
const utils_1 = require("./utils");
function prepareDebugAsync(bindings, apiNamespace) {
    let api;
    const resultPromise = new Promise((resolve, reject) => {
        api = { resumeExecution: resolve, failExecution: reject };
    });
    const extension = new Map();
    for (const key in bindings) {
        if (!bindings.hasOwnProperty(key)) {
            continue;
        }
        const getValue = bindings[key];
        try {
            getValue();
        }
        catch (error) {
            if (error instanceof ReferenceError) {
                // Variables can be declared after `await "debugger"` statement.
                continue;
            }
            throw error;
        }
        extension.set(key, getValue);
    }
    extension.set(apiNamespace, () => api);
    const startMessage = 'Async function is paused for debugging. The following variables ' +
        `are available: ${(0, utils_1.logKeys)(extension.keys())}.`;
    const stopMessage = 'Async function execution is resumed.';
    return { extension, resultPromise, api, startMessage, stopMessage };
}
exports.prepareDebugAsync = prepareDebugAsync;
